"""
openapigateway
==============

AWS CDK Construct that creates an Amazon API Gateway HttpApi based on a
parameterized OpenAPI 3 Document.
"""

__version__ = "0.2.0"

from openapigateway.constructs import OpenApiGateway
